==> walkers-fabric-1.18.2-4.5-sources.jar: tocraft/walkers/ability/impl/EvokerAbility.java <==
package tocraft.walkers.ability.impl;

import net.minecraft.class_1297;
import net.minecraft.class_1299;
import net.minecraft.class_1308;
import net.minecraft.class_1634;
import net.minecraft.class_1657;
import net.minecraft.class_1669;
import net.minecraft.class_1792;
import net.minecraft.class_1802;
import net.minecraft.class_1937;
import net.minecraft.class_2338;
import net.minecraft.class_2350;
import net.minecraft.class_243;
import net.minecraft.class_2680;
import net.minecraft.class_3218;
import tocraft.walkers.ability.ShapeAbility;

public class EvokerAbility<T extends class_1308> extends ShapeAbility<T> {

    @Override
    public void onUse(class_1657 player, T shape, class_1937 world) {
        // spawn vexes while sneaking
        if (player.method_18276() && world instanceof class_3218 serverLevel) {
            int i = 0;
            for (class_1297 entity : serverLevel.method_27909()) {
                if (entity instanceof class_1634 && player.method_5739(entity) <= 16)
                    ++i;
                if (i >= 8)
                    return;
            }

            for (int j = 0; j <= 2; j++) {
                class_1634 vex = new class_1634(class_1299.field_6059, world);
                vex.method_33574(player.method_19538());
                world.method_8649(vex);
            }
        } else {
            // Spawn 8 Evoker Fangs out from the player.
            class_243 origin = player.method_19538();
            class_243 facing = player.method_5720().method_18805(1, 0, 1); // fangs should not go up/down based on pitch

            // Iterate out 5 blocks
            for (int blockOut = 0; blockOut < 8; blockOut++) {
                origin = origin.method_1019(facing); // we add at the start -- no need to put a fang directly underneath the player!

                // Spawn an Evoker Fang at the given position.
                // For each position, we go up or down at most -+1 block per iteration.
                // If we cannot go up or down 1 block (or stay at the same level), the chain ends.

                // If the block underneath is solid, we are good to go.
                class_1669 fangs = new class_1669(world, origin.method_10216(), origin.method_10214(), origin.method_10215(), player.method_36454(), blockOut * 2, player);
                class_2338 underneathPosition = new class_2338(origin).method_10074();
                class_2680 underneath = world.method_8320(underneathPosition);
                if (underneath.method_26206(world, underneathPosition, class_2350.field_11036) && world.method_22347(underneathPosition.method_10084())) {
                    world.method_8649(fangs);
                    continue;
                }

                // Check underneath (2x down) again...
                class_2338 underneath2Position = new class_2338(origin).method_10087(2);
                class_2680 underneath2 = world.method_8320(underneath2Position);
                if (underneath2.method_26206(world, underneath2Position, class_2350.field_11036) && world.method_22347(underneath2Position.method_10084())) {
                    fangs.method_23327(fangs.method_23317(), fangs.method_23318() - 1, fangs.method_23321());
                    world.method_8649(fangs);
                    origin = origin.method_1031(0, -1, 0);
                    continue;
                }

                // Check above (1x up)
                class_2338 upPosition = new class_2338(origin).method_10084();
                class_2680 up = world.method_8320(underneath2Position);
                if (up.method_26206(world, upPosition, class_2350.field_11036) && world.method_22347(upPosition)) {
                    fangs.method_23327(fangs.method_23317(), fangs.method_23318() + 1, fangs.method_23321());
                    world.method_8649(fangs);
                    origin = origin.method_1031(0, 1, 0);
                    continue;
                }

                break;
            }
        }
    }

    @Override
    public class_1792 getIcon() {
        return class_1802.field_8687;
    }

    @Override
    public int getDefaultCooldown() {
        return 10;
    }
}

==> walkers-fabric-1.20.1-4.2-sources.jar: tocraft/walkers/ability/impl/EvokerAbility.java <==
package tocraft.walkers.ability.impl;

import net.minecraft.class_1297;
import net.minecraft.class_1299;
import net.minecraft.class_1308;
import net.minecraft.class_1634;
import net.minecraft.class_1657;
import net.minecraft.class_1669;
import net.minecraft.class_1792;
import net.minecraft.class_1802;
import net.minecraft.class_1937;
import net.minecraft.class_2338;
import net.minecraft.class_2350;
import net.minecraft.class_243;
import net.minecraft.class_2680;
import net.minecraft.class_3218;
import tocraft.walkers.ability.ShapeAbility;

public class EvokerAbility<T extends class_1308> extends ShapeAbility<T> {

    @Override
    public void onUse(class_1657 player, T shape, class_1937 world) {
        // spawn vexes while sneaking
        if (player.method_18276() && world instanceof class_3218 serverLevel) {
            int i = 0;
            for (class_1297 entity : serverLevel.method_27909()) {
                if (entity instanceof class_1634 && player.method_5739(entity) <= 16)
                    ++i;
                if (i >= 8)
                    return;
            }

            for (int j = 0; j <= 2; j++) {
                class_1634 vex = new class_1634(class_1299.field_6059, world);
                vex.method_33574(player.method_19538());
                world.method_8649(vex);
            }
        } else {
            // Spawn 8 Evoker Fangs out from the player.
            class_243 origin = player.method_19538();
            class_243 facing = player.method_5720().method_18805(1, 0, 1); // fangs should not go up/down based on pitch

            // Iterate out 5 blocks
            for (int blockOut = 0; blockOut < 8; blockOut++) {
                origin = origin.method_1019(facing); // we add at the start -- no need to put a fang directly underneath the player!

                // Spawn an Evoker Fang at the given position.
                // For each position, we go up or down at most -+1 block per iteration.
                // If we cannot go up or down 1 block (or stay at the same level), the chain ends.

                // If the block underneath is solid, we are good to go.
                class_1669 fangs = new class_1669(world, origin.method_10216(), origin.method_10214(), origin.method_10215(), player.method_36454(), blockOut * 2, player);
                class_2338 underneathPosition = class_2338.method_49638(origin).method_10074();
                class_2680 underneath = world.method_8320(underneathPosition);
                if (underneath.method_26206(world, underneathPosition, class_2350.field_11036) && world.method_22347(underneathPosition.method_10084())) {
                    world.method_8649(fangs);
                    continue;
                }

                // Check underneath (2x down) again...
                class_2338 underneath2Position = class_2338.method_49638(origin).method_10087(2);
                class_2680 underneath2 = world.method_8320(underneath2Position);
                if (underneath2.method_26206(world, underneath2Position, class_2350.field_11036) && world.method_22347(underneath2Position.method_10084())) {
                    fangs.method_23327(fangs.method_23317(), fangs.method_23318() - 1, fangs.method_23321());
                    world.method_8649(fangs);
                    origin = origin.method_1031(0, -1, 0);
                    continue;
                }

                // Check above (1x up)
                class_2338 upPosition = class_2338.method_49638(origin).method_10084();
                class_2680 up = world.method_8320(underneath2Position);
                if (up.method_26206(world, upPosition, class_2350.field_11036) && world.method_22347(upPosition)) {
                    fangs.method_23327(fangs.method_23317(), fangs.method_23318() + 1, fangs.method_23321());
                    world.method_8649(fangs);
                    origin = origin.method_1031(0, 1, 0);
                    continue;
                }

                break;
            }
        }
    }

    @Override
    public class_1792 getIcon() {
        return class_1802.field_8687;
    }

    @Override
    public int getDefaultCooldown() {
        return 10;
    }
}
